import nodemailer from 'nodemailer';
import type { Transporter, SendMailOptions } from 'nodemailer';
import { Report, Report as ReportSchema } from './models/report.js';

export interface Fault {
    coach: string;
    zone: string;
}

export interface Report {
    unitNumber: string;
    reported: string;
    comments: string;
    utcTimestamp: Date;
    faults: Fault[];
}

export async function handleFormData(data: any) {
    // Uploads form data to database and emails
    // to defined subscribers.

    const knownKeys = ['unitNumber', 'reported', 'comments', 'utcTimestamp'];

    const faults: Fault[] = [];

    const report: Report = {
        unitNumber: data.unitNumber,
        reported: data.reported,
        comments: data.comments || '',
        utcTimestamp: new Date(data.utcTimestamp),
        faults: []
    };

    for (const key in data) {
        if (!knownKeys.includes(key) && data[key] === 'on') {
            const match = key.match(/^(.+)-(.+)$/);
            if (match) {
                const [_, zone, coach] = match;
                report.faults.push({ coach, zone });
            }
        }
    }

    // sendMail(report); Disable sendMail in favour of daily,weekly,monthly reports
    sendToDatabase(report);
}

export async function fetchReports(): Promise<Report[]> {
    try {
        const reports = await ReportSchema.find();
        return reports;
    } catch (error) {
        console.error("Error fetching reports:", error);
        throw error;
    }
}

async function sendMail(report: Report): Promise<void> {
    const transporter = nodemailer.createTransport({
        host: process.env.SMTP_HOST,
        port: parseInt(process.env.SMTP_PORT || '587'),
        secure: false,
        auth: {
            user: process.env.SMTP_USER,
            pass: process.env.SMTP_PASS,
        }
    });

    const faultList = report.faults.map(f => `- ${f.zone} (${f.coach})`).join('\n');

    const mailOptions: SendMailOptions = {
        from: `"AC Reporter ${process.env.SMTP_USER}"`,
        to: process.env.MAIL_RECIPIENTS,
        subject: `New A/C Report: ${report.unitNumber}`,
        text: `A report has been submitted for unit ${report.unitNumber}.\n\n` +
              `Timestamp: ${report.utcTimestamp}\n\n` +
              `Faults:\n${faultList}\n\n` +
              `Reported to Maintenance: ${report.reported}\n\n` +
              `Comments:\n${report.comments || 'None'}`
    };

    try {
        const info = await transporter.sendMail(mailOptions);
        console.log('📧 Report sent:', info.messageId);
    } catch (err) {
        console.error('❌ Failed to send email:', err);
    }
}

async function sendToDatabase(report: Report) {
    const newReport = new ReportSchema({
        unitNumber: report.unitNumber,
        reported: report.reported,
        comments: report.comments,
        utcTimestamp: report.utcTimestamp,
        faults: report.faults
    });

    await newReport.save();

    console.log("Report saved to Database");
}